import React from 'react';

import useAppState from './useAppState';
import ApiHelper from '../helpers/ApiHelper';
import { useEffectOnce } from '../helpers/Hooks';


export const AppContext = React.createContext();

const AppContextProvider = props => {
  const [state, dispatch] = useAppState();
  const Api = new ApiHelper({ state });


  const getNetworkStats = () => {
    return Api.getNetworkStats()
      .then(networkStats => {
        dispatch({ type: 'UPDATE_NETWORK_STATS', networkStats });
      });
  };

  const getPoolConfig = () => {
    return Api.getPoolConfig()
      .then(config => {
        dispatch({ type: 'UPDATE_POOL_CONFIG', config });
      });
  };

  const getPoolStats = () => {
    return Api.getPoolStats()
      .then(stats => {
        dispatch({ type: 'UPDATE_POOL_STATS', stats });
      });
  };

  const actions = {
    getPoolConfig,
  };

  const setIntervals = () => {
    const { appSettings } = state;
    const intervals = [
      { fn: getPoolStats, time: appSettings.poolStatsUpdateInterval },
      { fn: getNetworkStats, time: appSettings.networkUpdateInterval },
    ];
    dispatch({ type: 'SET_INTERVALS', intervals });
  }

  useEffectOnce(() => {
    getPoolConfig().then(() =>
      getPoolStats().then(() =>
        getNetworkStats().then(() => {
          // initial API responses completed, app ready now
          setIntervals();
          dispatch({type: 'APP_LOADED', appLoaded: true});
        })
      )
    );
  })

  return (
    <AppContext.Provider value={{ state, actions }}>
      {state.appSettings.appLoaded && props.children}
    </AppContext.Provider>
  )
};

export default AppContextProvider;
